import ParenthesizedExtractor;
import MalformedDelimiterException;

import java.io.IOException;
import java.util.Hashtable;

public class Parser
	{
	/*
	Parse a Netscape bookmarks file and extract http references, folders,
	and any names the user attaches to them.

	A Netscape bookmarks file has the following format:
	A <DL> data listing tag starts a new folder, a </DL> tag ends the folder.
	Within a folder, a <DT> data term tag specifies an entry in that folder.
	That entry may be either a http reference or a nested folder. Folders may
	be arbitrarily nested.
	Any <DT> entry that does not start with "<A" is assumed to be a folder entry.
	All other html tags are ignored.

	Here are two typical <DT> entries:
	----------------------------------

	<DT>                      //start of the entry
	<A                        //start of the "A" tag
	HREF="http://www.unidata.ucar.edu/staff/caron/collab/wa_collab.html"
	ADD_DATE="895304949"      //date the user added this href
	LAST_VISIT="895219356"    //date the user last visited this href
	LAST_MODIFIED="895219356" //date the user last modified this bookmark entry
	>                         //closure of the "A" tag
	Wide Area Collaboration   //the user-assigned name of this href
	</A>                      //end of the "A" tag and end of the <DT> entry

	<DT>
	<H3
	FOLDED                    //folder entry
	ADD_DATE="882257499"      //date this folder was added
	>
	Guide Sections            //the user-assigned name of this folder
	</H3>
	*/

	/////////////////////////////////////////////////////////////////////////
	public static void main(String[] commandLineArguments)
		throws IOException
		{
		/*
		Tokenize a given bookmark file and extract its folders and http
		references.
		*/

		Hashtable symbolTable;
		if (commandLineArguments.length != 1)
			System.out.println("Parser: i need exactly one " +
				"bookmark file name as my argument.");
		else
			{
			try
				{
				symbolTable = extractTags(commandLineArguments[0]);
				}
			catch (MalformedDelimiterException ignored)
				{ //should never happen!
				System.out.println("internal error in Parser");
				ignored.printStackTrace();
				System.exit(1);
				}
			}
		}

	/////////////////////////////////////////////////////////////////////////
	public static Hashtable extractTags(String bookmarksFile)
		throws IOException, MalformedDelimiterException
		{
		/*
		Extract the html tags and any strings found into the Hashtable
		symbolTable. Return the symbol table.
		*/

		Hashtable symbolTable = new Hashtable();

		//create an extractor to extract the tags, collapse whitespace runs,
		//and ignore case in searches
		ParenthesizedExtractor extractor =
			new ParenthesizedExtractor(bookmarksFile, "<", ">", true, true);

		//get the first DT tag
		String DTTagContent = extractor.getNext("<DT>", ">");
		while (DTTagContent != null)
			{
			//get the DT tag's name
			String DTTagName = extractor.getUntil("<");

			//skip this DT tag if it isn't properly named
			if (DTTagName == null)
				{
				System.out.println("improperly terminated DT tag found");
				continue;
				}

			System.out.println();
			String endDTTag;
			//if the next tag is an A tag, it's a website, else it's a folder
			//(note that String.substring(i, j) returns the substring starting at
			//position i and ending at position j - 1)
			if (DTTagContent.substring(0, 2).toUpperCase().equals("<A"))
				{
				System.out.println("website found");
				//parse website content here
				endDTTag = extractor.getUntil("/A>");
				}
			else
				{
				System.out.println("folder found");
				//parse folder content here
				endDTTag = extractor.getUntil(">");
				}

			//skip this DT tag if it isn't properly terminated
			if (endDTTag == null)
				{
				System.out.println("improperly terminated DT tag found");
				continue;
				}

			System.out.println("Name = " + DTTagName);
			System.out.println("Content = " + DTTagContent.substring(1));

			//get the next DT tag's content
			DTTagContent = extractor.getNext("<DT>", ">");
			}

		return symbolTable;
		}

	/////////////////////////////////////////////////////////////////////////
	private static void parseFolderEntry(String folderEntry)
		throws MalformedDelimiterException
		{
		/*
		Extract the name and content of a folder entry in a bookmarks file.
		*/
		}

	/////////////////////////////////////////////////////////////////////////
	private static void parseWebsiteEntry(String websiteEntry)
		throws MalformedDelimiterException
		{
		/*
		Extract the name and content of a folder entry in a bookmarks file.
		*/
		}
	}
