
package com.knownspace.tools;

public class QuickSort
	{
	/*
	Sort an integer array.
	*/

	private static int[] intArray;

	//////////////////////////////////////////////////////////////////////
	public static int[] sort(int[] inputArray)
		{
		/*
		Sort a copy of an int array and return the sorted copy.
		*/

		//if there's nothing to do, just return
		if ((inputArray == null) || (inputArray.length <= 1))
			return inputArray;

		//make a copy and sort it, then return the sorted copy
		intArray = new int[inputArray.length];
		System.arraycopy(inputArray, 0, intArray, 0, inputArray.length);

		quickSort(intArray, 0, intArray.length - 1);

		return intArray;
		}

	//////////////////////////////////////////////////////////////////////
	public static void inplaceSort(int[] inputArray)
		{
		/*
		Sort a given int array in place (i.e. without making a copy).

		NOTE: should this method be synchronized for thread-safety?
		The problem is that it would cut down on speed for single-thread uses.
		*/

		//if there's nothing to do, just return
		if ((inputArray == null) || (inputArray.length <= 1))
			return;

		quickSort(inputArray, 0, inputArray.length - 1);
		}

	//////////////////////////////////////////////////////////////////////
	private static void quickSort(int[] intArray, int lo0, int hi0)
		{

		int lo = lo0;
		int hi = hi0;

		if ((hi - lo) <= 3)
			{
			bruteforceSort(intArray, lo, hi);
			return;
			}

		int mid = (lo + hi) / 2;
		while (lo < hi)
			{
			while ((lo < hi) && (intArray[lo] < intArray[mid]))
				lo++;
			while ((lo < hi) && (intArray[hi] >= intArray[mid]))
				hi--;
			if (lo < hi)
				swap(intArray, lo, hi);
			}

		if (hi < lo)
			{ int swap = lo; lo = hi; hi = swap; }

		quickSort(intArray, lo0, lo);

		if (lo == lo0)
			quickSort(intArray, lo + 1, hi0);
		else
			quickSort(intArray, lo, hi0);
		}
	
	//////////////////////////////////////////////////////////////////////
	private static final void bruteforceSort(int[] intArray, int lo, int hi)
		{
		/*
		Sort small numbers of consecutive intArray elements.
		*/

		int min, max;
		switch (hi - lo)
			{
			case 1:
				if (intArray[lo] > intArray[hi]) swap(intArray, lo, hi);
				return;

			case 2:
				min = lo;
				if (intArray[min] > intArray[lo + 1]) min = lo + 1;
				if (intArray[min] > intArray[lo + 2]) min = lo + 2;
				if (min != lo) swap(intArray, lo, min);
				bruteforceSort(intArray, lo + 1, hi);
				return;

			case 3:
				min = lo;
				if (intArray[min] > intArray[lo + 1]) min = lo + 1;
				if (intArray[min] > intArray[lo + 2]) min = lo + 2;
				if (intArray[min] > intArray[lo + 3]) min = lo + 3;
				if (min != lo) swap(intArray, lo, min);
				max = hi;
				if (intArray[max] < intArray[hi - 1]) max = hi - 1;
				if (intArray[max] < intArray[hi - 2]) max = hi - 2;
				if (max != hi) swap(intArray, max, hi);
				bruteforceSort(intArray, lo + 1, hi - 1);
				return;
			}
		}

	//////////////////////////////////////////////////////////////////////
	private static final void swap(int[] intArray, int index1, int index2)
		{
		/*
		Swap the values in intArry[index1] and intArray[index2].
		*/

		int swap = intArray[index1];
		intArray[index1] = intArray[index2];
		intArray[index2] = swap;
		}
	}
